# clang static analysis
%bcond_with clang_sa

%global sysctl_prio       50
%global sysctl_optmem_max 81920
%global distroname_ext    %{_vendor}
%global fipscheck_evr     1.5.0-9
%global hmaccalc_evr      0.9.14-10%{?dist}


Summary:        User-space to access the Linux kernel crypto API
Name:           libkcapi
Version:        1.4.0
Release:        7%{?dist}
License:        BSD or GPLv2
URL:            https://www.chronox.de/%{name}.html
Source0:        https://www.chronox.de/%{name}/%{name}-%{version}.tar.xz

BuildRequires:  coreutils gcc make libtool hardlink
BuildRequires:  openssl systemd xmlto
BuildRequires:  kernel-headers >= 4.10.0
%if %{with clang_sa}
BuildRequires:  clang
%endif
Requires:       systemd


%description
The Linux kernel exports a Netlink interface of type AF_ALG
to allow user space to utilize the kernel crypto API.

libkcapi uses this Netlink interface and exports easy to use APIs
so that a developer does not need to consider the low-level Netlink interface handling.

The library does not implement any cipher algorithms.
All consumer requests are sent to the kernel for processing.
Results from the kernel crypto API are returned to the consumer via the library API.

The kernel interface and therefore this library can be used by unprivileged processes.

The focus during the development of this library is put on speed.
This library does not perform any memcpy for processing the cryptographic data!
The library uses scatter / gather lists to eliminate the need for moving data around in memory.


%package        tools
Summary:        Utility applications for the %{name} package
Requires:       %{name} == %{version}-%{release}

%description    tools
Utility applications that are provided with %{name}.  This includes
tools to use message digests, symmetric ciphers and random number
generators implemented in the Linux kernel from command line.


%package        fipscheck
Summary:        Drop-in replacements for fipscheck/fipshmac provided by libkcapi
Requires:       %{name}   == %{version}-%{release}
Provides:       fipscheck == %{fipscheck_evr}.1

%description    fipscheck
Provides drop-in replacements for fipscheck and fipshmac tools
(from package fipscheck) using libkcapi.


%package        hmaccalc
Summary:        Drop-in replacements for hmaccalc provided by libkcapi
Requires:       %{name}   == %{version}-%{release}
Provides:       hmaccalc  == %{hmaccalc_evr}.1

%description    hmaccalc
Provides drop-in replacements for sha*hmac tools
(from package hmaccalc) using libkcapi.


%package        devel
Summary:        Development files for libkcapi
Requires:       %{name} == %{version}-%{release}

%description    devel
Header files for applications that use libkcapi.


%package        static
Summary:        Static library for -static linking with libkcapi
Requires:       %{name}-devel == %{version}-%{release}

%description    static
This package contains the libkcapi static libraries for -static linking.


%package        tests
Summary:        Testing scripts for libkcapi
Requires:       %{name}       == %{version}-%{release}
Requires:       %{name}-tools == %{version}-%{release}
Requires:       %{name}-hmaccalc == %{version}-%{release}
Requires:       coreutils openssl perl-interpreter

%description    tests
Auxiliary scripts for testing libkcapi.


%prep
%autosetup

cat << EOF > README.%{distroname_ext}
This package increases the default limit of the ancillary buffer size
per kernel socket defined in \`net.core.optmem_max\` to %{sysctl_optmem_max} bytes.

For this preset to become active it requires a reboot after the
installation of this package.  You can also manually increase this
limit by invocing \`sysctl net.core.optmem_max=%{sysctl_optmem_max}\` as the
super-user, e.g. using \`su\` or \`sudo\` on the terminal.

This is done to provide consumers of the new Linux Kernel Crypto API
User Space Interface a well sufficient and reasonable maximum limit
by default, especially when using AIO with a larger amount of IOVECs.

For further information about the AF_ALG kernel socket and AIO, see
the discussion at the kernel-crypto mailing-list:
https://www.mail-archive.com/linux-crypto@vger.kernel.org/msg30417.html

See the instructions given in '%{_sysctldir}/50-default.conf',
if you need or want to override the preset made by this package.
EOF

cat << EOF > %{sysctl_prio}-%{name}-optmem_max.conf
# See the 'README.%{distroname_ext}' file shipped in %%doc
# with the %{name} package.
#
# See '%{_sysctldir}/50-default.conf',
# if you need or want to override this preset.

# Increase the ancillary buffer size per socket.
net.core.optmem_max = %{sysctl_optmem_max}
EOF


%build
autoreconf -fiv
%configure               \
  --libdir=/%{_lib}      \
  --disable-silent-rules \
  --enable-kcapi-encapp  \
  --enable-kcapi-dgstapp \
  --enable-kcapi-hasher  \
  --enable-kcapi-rngapp  \
  --enable-kcapi-speed   \
  --enable-kcapi-test    \
  --enable-shared        \
  --enable-static        \
  --enable-sum-prefix=   \
  --enable-sum-dir=/%{_lib} \
  --with-pkgconfigdir=%{_libdir}/pkgconfig

%make_build all man


%install
%make_install

# Install sysctl.d preset.
mkdir -p %{buildroot}%{_sysctldir}
install -Dpm 0644 -t %{buildroot}%{_sysctldir} %{sysctl_prio}-%{name}-optmem_max.conf

mkdir -p %{buildroot}%{_pkgdocdir}
install -Dpm 0644 -t %{buildroot}%{_pkgdocdir} README.%{distroname_ext} README.md CHANGES.md TODO

rm -f %{buildroot}%{_bindir}/md5sum %{buildroot}%{_bindir}/sha*sum %{buildroot}%{_bindir}/sm*sum

find %{buildroot} -type f -name '*.la' -print -delete
find %{buildroot} -type f -name '*.hmac' -print -delete
find %{buildroot} -type f -size 0 -print -delete
find %{buildroot}%{_pkgdocdir} -type f -print | xargs chmod -c 0644
find %{buildroot}%{_pkgdocdir} -type d -print | xargs chmod -c 0755

%global apps_hmaccalc sha1hmac sha224hmac sha256hmac sha384hmac sha512hmac sm3hmac
%global apps_fipscheck sha1sum sha224sum sha256sum sha384sum sha512sum md5sum sm3sum fipscheck fipshmac
for app in %{apps_hmaccalc}; do
    if test -e %{buildroot}%{_bindir}/$app; then
        { bin/kcapi-hasher -n sha512hmac %{buildroot}%{_bindir}/$app || exit 1; } | cut -f 1 -d ' ' > %{buildroot}/%{_lib}/hmaccalc/$app.hmac
    fi
done
for app in %{apps_fipscheck}; do
    if test -e %{buildroot}%{_bindir}/$app; then
        bin/kcapi-hasher -n fipshmac -d %{buildroot}/%{_lib}/fipscheck %{buildroot}%{_bindir}/$app || exit 1
    fi
done
hardlink -cfv %{buildroot}%{_bindir}
bin/kcapi-hasher -n fipshmac -d %{buildroot}/%{_lib}/fipscheck   %{buildroot}/%{_lib}/libkcapi.so.%{version} || exit 1
ln -s libkcapi.so.%{version}.hmac    %{buildroot}/%{_lib}/fipscheck/libkcapi.so.%(echo %{version} | cut -d . -f 1).hmac


%check
%if %{with clang_sa}
%make_build scan
%endif

pushd test
ENABLE_FUZZ_TEST=1 \
NO_32BIT_TEST=1 \
./test-invocation.sh || :
popd


%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig


%files
%license COPYING*
%doc %dir %{_pkgdocdir}
%doc %{_pkgdocdir}/README.md
%doc %{_pkgdocdir}/README.%{distroname_ext}
%{_sysctldir}/%{sysctl_prio}-%{name}-optmem_max.conf
/%{_lib}/%{name}.so.*
/%{_lib}/fipscheck/%{name}.so.*.hmac


%files tools
%{_bindir}/kcapi*
%{_mandir}/man1/kcapi*.1.*

%files fipscheck
%{_bindir}/fips*
/%{_lib}/fipscheck/fips*.hmac


%files hmaccalc
%{_bindir}/sha*hmac
%{_bindir}/sm*hmac
/%{_lib}/hmaccalc/sha*hmac.hmac
/%{_lib}/hmaccalc/sm*hmac.hmac


%files devel
%doc %{_pkgdocdir}/CHANGES.md
%doc %{_pkgdocdir}/TODO
%{_includedir}/kcapi.h
/%{_lib}/%{name}.so
%{_libdir}/pkgconfig/%{name}.pc
%{_mandir}/man3/kcapi_*.3.*


%files static
/%{_lib}/%{name}.a


%files tests
%{_libexecdir}/%{name}/*


%changelog
* Thu Sep 26 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.4.0-7
- Rebuilt for clarifying the packages requirement in BaseOS and AppStream

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.4.0-6
- Rebuilt for loongarch release

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.4.0-5
- Rebuilt for OpenCloudOS Stream 23.09

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.4.0-4
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 1.4.0-3
- Rebuilt for OpenCloudOS Stream 23

* Thu Nov 10 2022 rockerzhu <rockerzhu@tencent.com> - 1.4.0-2
- add subpackages fipscheck and hmaccalc

* Tue Apr 26 2022 rockerzhu <rockerzhu@tencent.com> - 1.4.0-1
- Initial build
